h1,h3{
    line-height:1
}
address,blockquote,dfn,em{
    font-style:italic
}
html{
    font-size:100.01%
}
body{
    font-size:75%;
    color:#222;
    background:#fff;
    font-family:\"Helvetica Neue\",Arial,Helvetica,sans-serif
}
h1,h2,h3,h4,h5,h6{
    font-weight:400;
    color:#111
}
h1{
    font-size:3em;
    margin-bottom:.5em
}
h2{
    font-size:2em;
    margin-bottom:.75em
}
h3{
    font-size:1.5em;
    margin-bottom:1em
}
h4{
    font-size:1.2em;
    line-height:1.25;
    margin-bottom:1.25em
}
h5,h6{
    font-size:1em;
    font-weight:700
}
h5{
    margin-bottom:1.5em
}
h1 img,h2 img,h3 img,h4 img,h5 img,h6 img{
    margin:0
}
p{
    margin:0 0 1.5em
}
.left{
    float:left!important
}
p .left{
    margin:1.5em 1.5em 1.5em 0;
    padding:0
}
.right{
    float:right!important
}
p .right{
    margin:1.5em 0 1.5em 1.5em;
    padding:0
}
address,dl{
    margin:0 0 1.5em
}
a:focus,a:hover{
    color:#09f
}
a{
    color:#06c;
    text-decoration:underline
}
.quiet,blockquote,del{
    color:#666
}
blockquote{
    margin:1.5em
}
dfn,dl dt,strong,th{
    font-weight:700
}
sub,sup{
    line-height:0
}
abbr,acronym{
    border-bottom:1px dotted #666
}
pre{
    margin:1.5em 0;
    white-space:pre
}
code,pre,tt{
    font:1em 'andale mono','lucida console',monospace;
    line-height:1.5
}
li ol,li ul{
    margin:0
}
ol,ul{
    margin:0 1.5em 1.5em 0;
    padding-left:1.5em
}
ul{
    list-style-type:disc
}
ol{
    list-style-type:decimal
}
dd{
    margin-left:1.5em
}
table{
    margin-bottom:1.4em;
    width:100%
}
thead th{
    background:#c3d9ff
}
caption,td,th{
    padding:4px 10px 4px 5px
}
tbody tr.even td,tbody tr:nth-child(even) td{
    background:#e5ecf9
}
tfoot{
    font-style:italic
}
caption{
    background:#eee
}
.small{
    font-size:.8em;
    margin-bottom:1.875em;
    line-height:1.875em
}
.large{
    font-size:1.2em;
    line-height:2.5em;
    margin-bottom:1.25em
}
.hide{
    display:none
}
.loud{
    color:#000
}
.highlight{
    background:#ff0
}
.added{
    background:#060;
    color:#fff
}
.removed{
    background:#900;
    color:#fff
}
.first{
    margin-left:0;
    padding-left:0
}
.last{
    margin-right:0;
    padding-right:0
}
.top{
    margin-top:0;
    padding-top:0
}
.bottom{
    margin-bottom:0;
    padding-bottom:0
}
